#!/usr/bin/env python3

import csv
import datetime as dt

import argparse

from numpy import std

parser = argparse.ArgumentParser()
parser.add_argument('--filter', type=int, default=0, help='number of RFC needed as country to be included')
parser.add_argument('inputFile', type=argparse.FileType('r'))
args = parser.parse_args()

inFile = args.inputFile

# outer scope for quotas
countriesAll, yearsAll, yearmonthsAll = {}, {}, {}
countriesCompany, yearsCompany, yearmonthsCompany = {}, {}, {}
countriesStd, yearsStd, yearmonthsStd = {}, {}, {}

# helper list to check if month is valid
months = list()
for x in range(1, 13):
    months.append(dt.date(2000, x, 1).strftime('%b').lower())

# input columns
columns = ['document', 'authorname', 'company', 'country', 'pages', 'month', 'year', 'day', 'features']

# build different files for all combinations
outFilesAll, outFilesCompany, outFilesCompQuota, outFilesStdQuota = list(), list(), list(), list()
for span in ('sum', 'yearly', 'monthly'):
    for case in ('-dirty', '-clean', '-errors'):
        x = span + case
        outFilesAll.append(x + '.csv')
        outFilesCompany.append(x + '-company.csv')
        outFilesCompQuota.append(x + '-company-quota.csv')


def process(case):
    """
    iterates over input csv and builds country stats
    @param case 'company' if only employees should be counted, else 'all'
    """
    years, yearmonths, countries, companies = dict(), dict(), dict(), dict()


    inFile.seek(0)
    for row in csv.DictReader(inFile, fieldnames=columns, delimiter=':'):
        if row['year'] == '2020':
            continue
        if case == 'company':
            if row['company'] is None or row['company'] == 'UNKNOWN':
                continue
            company = row['company']
            companies[company] = 1 if company not in companies else companies[company] + 1
        # obtain and sanitize inputs
        year = row['year'] if row['year'] is not None else 'NONE'
        month = row['month'] if row['month'] is not None else 'NONE'
        month = dt.datetime.strptime(month, '%b').month if month in months else month
        yearmonth = year + '-' + '0' + str(month) + '-01' if isinstance(month, int) and month < 10 else year + '-' + str(month) + '-01'
        country = row['country'] if row['country'] is not None else 'NONE'
        # build sum dict
        countries[country] = 1 if country not in countries else countries[country] + 1
        # build year/month superdicts with subdicts
        if year not in years:
            years[year] = {country: 1}
        elif country not in years[year]:
            years[year][country] = 1
        else:
            years[year][country] += 1
        if yearmonth not in yearmonths:
            yearmonths[yearmonth] = {country: 1}
        elif country not in yearmonths[yearmonth]:
            yearmonths[yearmonth][country] = 1
        else:
            yearmonths[yearmonth][country] += 1

    if case == 'company':
        companies = {k: v for k, v in companies.items() if v >= args.filter}
    countries = {k: v for k, v in countries.items() if v >= args.filter}
    for year, yearDict in years.items():
        yearDict = {k: v for k, v in yearDict.items() if k in countries.keys()}
        years[year] = yearDict
    for yearmonth, yearmonthDict in yearmonths.items():
        yearmonthDict = {k: v for k, v in yearmonthDict.items() if k in countries.keys()}
        yearmonths[yearmonth] = yearmonthDict

    # update outer scope vars used to calculate quotas
    if case == 'all':
        outFiles = outFilesAll
        yearsAll.update(years)
        yearmonthsAll.update(yearmonths)
        countriesAll.update(countries)
    elif case == 'company':
        outFiles = outFilesCompany
        yearsCompany.update(years)
        yearmonthsCompany.update(yearmonths)
        countriesCompany.update(countries)

        with open('company.csv', 'w') as outFile:
            outFile.write('company,amount,share,concentration\n')
            tuple_list = list()
            var_list = list()
            total = 0
            concentration = 1
            for company, amount in companies.items():
                tuple_list.append((company, amount))
                var_list.append(amount)
                total += amount
            for tup in sorted(tuple_list, key=lambda tup: tup[1], reverse=True):
                share = round(tup[1]/total, 12)
                outFile.write(tup[0]+','+str(tup[1])+','+str(share)+','+str(concentration)+'\n')
                concentration = round(concentration - share, 12)
            outFile.write('sum,' + str(total) + ',,\n')
            outFile.write('std,' + str(round(std(var_list))) + ',,\n')

    # write total stats
    with open(outFiles[0], 'w') as outFile:
        outFile.write('country,amount,share,concentration\n')
        tuple_list = list()
        var_list = list()
        total = 0
        concentration = 1
        for country, amount in countries.items():
            tuple_list.append((country, amount))
            var_list.append(amount)
            total += amount
        for tup in sorted(tuple_list, key=lambda tup: tup[1], reverse=True):
            share = round(tup[1]/total, 12)
            outFile.write(tup[0]+','+str(tup[1])+','+str(share)+','+str(concentration)+'\n')
            concentration = round(concentration - share, 12)
        outFile.write('sum,' + str(total) + ',,\n')
        outFile.write('std,' + str(round(std(var_list))) + ',,\n')

    # write yearly stats
    with open(outFiles[3], 'w') as outFile:
        outFile.write('date,country,amount\n')
        for year, yearcountries in sorted(years.items()):
            for country, amount in sorted(yearcountries.items()):
                outFile.write(year+','+country+','+str(amount)+'\n')

    # write monthly stats
    with open(outFiles[6], 'w') as outFile:
        outFile.write('date,country,amount\n')
        for yearmonth, yearmonthcountries in sorted(yearmonths.items()):
            for country, amount in sorted(yearmonthcountries.items()):
                outFile.write(yearmonth+','+country+','+str(amount)+'\n')

    # write sanitized total stats
    with open(outFiles[1], 'w') as outFile:
        with open(outFiles[2], 'w') as outFileErrors:
            outFile.write('country,amount,share,concentration\n')
            outFileErrors.write('country,amount\n')
            tuple_listC, tuple_listE = list(), list()
            var_list = list()
            totalC, totalE = 0, 0
            concentration = 1
            for country, amount in countries.items():
                if country not in ['UNKNOWN', 'NONE']:
                    tuple_listC.append((country, amount))
                    var_list.append(amount)
                    totalC += amount
                else:
                    tuple_listE.append((country, amount))
                    totalE += amount
            for tup in sorted(tuple_listC, key=lambda tup: tup[1], reverse=True):
                share = round(tup[1]/totalC, 12)
                outFile.write(tup[0]+','+str(tup[1])+','+str(share)+','+str(concentration)+'\n')
                concentration = round(concentration - share, 12)
            outFile.write('sum,' + str(totalC) + ',,\n')
            outFile.write('std,' + str(round(std(var_list))) + ',,\n')
            for tup in sorted(tuple_listE, key=lambda tup: tup[1], reverse=True):
                outFileErrors.write(tup[0]+','+str(tup[1])+'\n')
            outFileErrors.write('sum,' + str(totalE) + '\n')

    # write sanitized yearly stats
    with open(outFiles[4], 'w') as outFile:
        with open(outFiles[5], 'w') as outFileErrors:
            outFile.write('date,country,amount\n')
            outFileErrors.write('date,country,amount\n')
            for year, yearcountries in sorted(years.items()):
                if year not in ['UNKNOWN', 'NONE']:
                    for country, amount in sorted(yearcountries.items()):
                        if country not in ['UNKNOWN', 'NONE']:
                            outFile.write(year+','+country+','+str(amount)+'\n')
                        else:
                            outFileErrors.write(year+','+country+','+str(amount)+'\n')
                else:
                    for country, amount in sorted(yearcountries.items()):
                        outFileErrors.write(year+','+country+','+str(amount)+'\n')

    # write sanitized monthly stats
    with open(outFiles[7], 'w') as outFile:
        with open(outFiles[8], 'w') as outFileErrors:
            outFile.write('date,country,amount\n')
            outFileErrors.write('date,country,amount\n')
            for yearmonth, yearmonthcountries in sorted(yearmonths.items()):
                if 'UNKNOWN' not in yearmonth and 'NONE' not in yearmonth:
                    for country, amount in sorted(yearmonthcountries.items()):
                        if country not in ['UNKNOWN', 'NONE']:
                            outFile.write(yearmonth+','+country+','+str(amount)+'\n')
                        else:
                            outFileErrors.write(yearmonth+','+country+','+str(amount)+'\n')
                else:
                    for country, amount in sorted(yearmonthcountries.items()):
                        outFileErrors.write(yearmonth+','+country+','+str(amount)+'\n')


# process country stats
process('all')
process('company')


# calculate and write total stats
with open(outFilesCompQuota[0], 'w') as outFile:
    outFile.write('country,quota\n')
    tuple_list = list()
    total = list()
    for country, amount in countriesAll.items():
        if country in countriesCompany:
            quota = countriesCompany[country] / amount
        else:
            quota = 0
        tuple_list.append((country, quota))
    for tup in sorted(tuple_list, key=lambda tup: tup[1], reverse=True):
        outFile.write(tup[0]+','+str(tup[1])+'\n')
        total.append(tup[1])
    if total:
        outFile.write('avg,' + str(sum(total)/len(total)) + '\n')
        outFile.write('std,' + str(std(total)))

# calculate and write yearly stats
with open(outFilesCompQuota[3], 'w') as outFile:
    outFile.write('date,country,quota\n')
    for year, yearcountries in sorted(yearsAll.items()):
        if year in yearsCompany:
            for country, amount in sorted(yearcountries.items()):
                if country in yearsCompany[year]:
                    quota = yearsCompany[year][country] / amount
                    outFile.write(year+','+country+','+str(quota)+'\n')
                else:
                    outFile.write(year+','+country+',0\n')
        else:
            for country, amount in sorted(yearcountries.items()):
                outFile.write(year+','+country+',0\n')

# calculate and write monthly stats
with open(outFilesCompQuota[6], 'w') as outFile:
    outFile.write('date,country,quota\n')
    for yearmonth, yearmonthcountries in sorted(yearmonthsAll.items()):
        if yearmonth in yearmonthsCompany:
            for country, amount in sorted(yearmonthcountries.items()):
                if country in yearmonthsCompany[yearmonth]:
                    quota = yearmonthsCompany[yearmonth][country] / amount
                    outFile.write(yearmonth+','+country+','+str(quota)+'\n')
                else:
                    outFile.write(yearmonth+','+country+',0\n')
        else:
            for country, amount in sorted(yearmonthcountries.items()):
                outFile.write(yearmonth+','+country+',0\n')

# calculate and write sanitized total stats
with open(outFilesCompQuota[1], 'w') as outFile:
    with open(outFilesCompQuota[2], 'w') as outFileErrors:
        outFile.write('country,quota\n')
        outFileErrors.write('country,amount\n')
        tuple_list = list()
        totalC, totalE = list(), list()
        for country, amount in countriesAll.items():
            if country in countriesCompany:
                quota = countriesCompany[country] / amount
            else:
                quota = 0
            tuple_list.append((country, quota))
        for tup in sorted(tuple_list, key=lambda tup: tup[1], reverse=True):
            if tup[0] not in ['UNKNOWN', 'NONE']:
                outFile.write(tup[0]+','+str(tup[1])+'\n')
                totalC.append(tup[1])
            else:
                outFileErrors.write(tup[0]+','+str(amount)+'\n')
                totalE.append(tup[1])
        if totalC:
            outFile.write('avg,' + str(sum(totalC)/len(totalC)) + '\n')
            outFile.write('std,' + str(std(total)))
        if totalE:
            outFileErrors.write('avg,' + str(sum(totalE)/len(totalE)) + '\n')

# calculate and write sanitized yearly stats
with open(outFilesCompQuota[4], 'w') as outFile:
    with open(outFilesCompQuota[5], 'w') as outFileErrors:
        outFile.write('date,country,quota\n')
        outFileErrors.write('date,country,amount\n')
        for year, yearcountries in sorted(yearsAll.items()):
            if year not in ['UNKNOWN', 'NONE']:
                if year in yearsCompany:
                    for country, amount in sorted(yearcountries.items()):
                        if country not in ['UNKNOWN', 'NONE']:
                            if country in yearsCompany[year]:
                                quota = yearsCompany[year][country] / amount
                                outFile.write(year+','+country+','+str(quota)+'\n')
                            else:
                                outFile.write(year+','+country+',0\n')
                        else:
                            if country in yearsCompany[year]:
                                quota = yearsCompany[year][country] / amount
                                outFileErrors.write(year+','+country+','+str(quota)+'\n')
                            else:
                                outFileErrors.write(year+','+country+',0\n')
                # no match for whole year
                else:
                    for country, amount in sorted(yearcountries.items()):
                        if country not in ['UNKNOWN', 'NONE']:
                            outFile.write(year+','+country+',0\n')
                        else:
                            outFileErrors.write(year+','+country+',0\n')
            # year is undefined -> all into errorlog
            else:
                for country, amount in sorted(yearcountries.items()):
                    outFileErrors.write(year+','+country+','+str(amount)+'\n')

# calculate and write sanitized monthly stats
with open(outFilesCompQuota[7], 'w') as outFile:
    with open(outFilesCompQuota[8], 'w') as outFileErrors:
        outFile.write('date,country,quota\n')
        outFileErrors.write('date,country,amount\n')
        for yearmonth, yearmonthcountries in sorted(yearmonthsAll.items()):
            if 'UNKNOWN' not in yearmonth and 'NONE' not in yearmonth:
                if yearmonth in yearmonthsCompany:
                    for country, amount in sorted(yearmonthcountries.items()):
                        if country not in ['UNKNOWN', 'NONE']:
                            if country in yearmonthsCompany[yearmonth]:
                                quota = yearmonthsCompany[yearmonth][country] / amount
                                outFile.write(yearmonth+','+country+','+str(quota)+'\n')
                            else:
                                outFile.write(yearmonth+','+country+',0\n')
                        else:
                            if country in yearmonthsCompany[yearmonth]:
                                quota = yearmonthsCompany[yearmonth][country] / amount
                                outFileErrors.write(yearmonth+','+country+','+str(quota)+'\n')
                            else:
                                outFileErrors.write(yearmonth+','+country+',0\n')
                # no match for whole year-month
                else:
                    for country, amount in sorted(yearmonthcountries.items()):
                        if country not in ['UNKNOWN', 'NONE']:
                            outFile.write(yearmonth+','+country+',0\n')
                        else:
                            outFileErrors.write(yearmonth+','+country+',0\n')
            # year-month is undefined -> all into errorlog
            else:
                for country, amount in sorted(yearmonthcountries.items()):
                    outFileErrors.write(yearmonth+','+country+','+str(amount)+'\n')
