#!/usr/bin/env python3

import csv
import sys
import argparse
import math

from os import makedirs

from numpy import std

parser = argparse.ArgumentParser()
parser.add_argument('--step', type=float, help='approx span of histogram classes')
parser.add_argument('inputFile', type=argparse.FileType('r'))
args = parser.parse_args()

inFile = args.inputFile

if not inFile:
    print('histogram.py: ERR: No input given, returning')
    sys.exit(1)
else:
    inputFile = inFile.name

# build out dirs
outDir = '/'.join(['out', *inputFile.split('/')[2:-1], inputFile.split('/')[-1][:len(inputFile.split('/')[-1]) - 4]])
makedirs(outDir, exist_ok=True)

def roundup(a, digits=0):
    n = 10**-digits
    return round(math.ceil(a / n) * n, digits)

vals = list()

with open(inputFile, 'r') as inFile:
    for row in csv.DictReader(inFile):
        if row['country'] == 'avg' or row['country'] == 'sum' or row['country'] == 'std':
            continue
        val = float(row['quota'])
        vals.append(val)

vals.sort()

if args.step:
    normStep = args.step
else:
    normStep = std(vals) / 2


# if vals[-1] + normStep > 1:
#     fit = (vals[0] + normStep * len(vals) - 1) / len(vals)
#     step = normStep - fit
# else:
step = normStep

with open(outDir + '/values', 'w') as outFile:
    print(*vals, sep=' ', file=outFile)

with open(outDir + '/histogram.csv', 'w') as outFile:
    outFile.write('lw,hw,amount\n')
    i = 0
    x = vals[0]
    y = roundup(x + step, 5)
    a = len([v for v in vals if v >= x and v < y])
    outFile.write(str(x) + ',' + str(y) + ',' + str(a) + '\n')
    while y < 1:
        x = y
        y = roundup(x + step, 5)
        a = len([v for v in vals if v >= x and v < y])
        outFile.write(str(x) + ',' + str(y) + ',' + str(a) + '\n')
        if y > vals[-1]:
            sys.exit(0)
        i += 1
