#!/usr/bin/env python3

import csv
import sys
import argparse

from os import makedirs
from collections import OrderedDict
from operator import itemgetter

from numpy import std

parser = argparse.ArgumentParser()
parser.add_argument('--classinterval', type=int, default=1, help='span of equivalence classes')
parser.add_argument('inputFile', type=argparse.FileType('r'))
args = parser.parse_args()

inFile = args.inputFile

case = ''
# check input
if not inFile:
    print('plotbuild.py: ERR: No input given, returning')
    sys.exit(1)
else:
    inputFile = inFile.name
    if 'sum' in inputFile:
        case = 'amount'
    elif 'quota' in inputFile:
        case = 'quota'
    elif 'yearly' in inputFile:
        case = 'year'
    elif 'monthly' in inputFile:
        case = 'year-month'
    elif 'company' in inputFile:
        case = 'company'
    else:
        print('plotbuild.py: ERR: malformed input, returning\n\t' + inputFile)
        sys.exit(1)

# build out dirs
outDir = 'out/' + inputFile.split('/')[-2] + '/' + inputFile.split('/')[-1][:len(inputFile.split('/')[-1]) - 4]
makedirs(outDir, exist_ok=True)

spans = list()
countries = set()

if 'yearly' in inputFile:
    for year in range(1968, 2020):
        spans.append(str(year))
else:
    for year in range(1968, 2020):
        for month in range(1, 13):
            spans.append(str(year) + f'-{month:02}' + '-01')


# build country statistics
if case in ('year', 'year-month'):

    # init spans and countries gapless
    first_rfc = OrderedDict()

    spanSums = OrderedDict.fromkeys(spans, 0)

    # read first for countries, second for values
    with open(inputFile, 'r') as inFile:
        for row in csv.DictReader(inFile):
            countries.add(row['country'])
        countrySuperDict = dict.fromkeys(countries)
        for country in countries:
            countrySuperDict[country] = OrderedDict.fromkeys(spans, 0)
        inFile.seek(0)
        for row in csv.DictReader(inFile):
            countrySuperDict[row['country']][row['date']] = row['amount']

    # count absolute
    for country, countryDict in countrySuperDict.items():
        for span, amount in countryDict.items():
            # get first occurence
            amount = int(amount)
            if amount != 0 and country not in first_rfc.keys():
                first_rfc[country] = span
            spanSums[span] += amount
            countryDict[span] = str(amount)
        outputFile = outDir + '/' + country.replace(' ', '_') + '-absolute.csv'
        with open(outputFile, 'w') as outFile:
            outFile.write('date' + ',amount\n')
            first = False
            lastSpan = None
            for span, amount in countryDict.items():
                if not first:
                    if int(amount) > 0:
                        if lastSpan is not None:
                            outFile.write(lastSpan+',0\n')
                        first = True
                    else:
                        lastSpan = span
                        continue
                outFile.write(span+','+amount+'\n')
    with open(outDir + '/sum-absolute.csv', 'w') as outFile:
        outFile.write('date' + ',amount\n')
        for span, amount in spanSums.items():
            outFile.write(span+','+str(amount)+'\n')

    # calculate relative
    for country, countryDict in countrySuperDict.items():
        outputFile = outDir + '/' + country.replace(' ', '_') + '-relative.csv'
        with open(outputFile, 'w') as outFile:
            outFile.write('date' + ',quota\n')
            first = False
            lastSpan = None
            for span, amount in countryDict.items():
                if not first:
                    if int(amount) > 0:
                        if lastSpan is not None:
                            outFile.write(lastSpan+',0\n')
                        first = True
                    else:
                        lastSpan = span
                        continue
                if spanSums[span] == 0:
                    continue
                elif int(amount) == 0:
                    outFile.write(span+',0\n')
                else:
                    outFile.write(span+','+str(int(amount)/spanSums[span])+'\n')

    # sum up
    for country, countryDict in countrySuperDict.items():
        amountPrev = 0
        for span, amount in countryDict.items():
            amountNew = int(amount) + amountPrev
            countryDict[span] = str(amountNew)
            amountPrev = amountNew
        outputFile = outDir + '/' + country.replace(' ', '_') + '-summed.csv'
        with open(outputFile, 'w') as outFile:
            outFile.write('date' + ',amount\n')
            first = False
            lastSpan = None
            for span, amount in countryDict.items():
                if not first:
                    if int(amount) > 0:
                        if lastSpan is not None:
                            outFile.write(lastSpan+',0\n')
                        first = True
                    else:
                        lastSpan = span
                        continue
                outFile.write(span+','+amount+'\n')
    with open(outDir + '/sum-summed.csv', 'w') as outFile:
        outFile.write('date' + ',amount\n')
        amountPrev = 0
        for span, amount in spanSums.items():
            amountNew = int(amount) + amountPrev
            outFile.write(span+','+str(amountNew)+'\n')
            amountPrev = amountNew

    # calculate occurence quota
    tuple_list = list()
    vals = list()
    for country in first_rfc.keys():
        if first_rfc[country] is None:
            continue
        quota = round(spans.index(first_rfc[country])/len(spans), 12)
        tuple_list.append((country, quota))
        vals.append(quota)
    tuple_list.sort(key=itemgetter(1, 0), reverse=True)
    with open(outDir + '/occurence_quota.csv', 'w') as outFile:
        outFile.write('country,date,quota\n')
        for tup in sorted(tuple_list, key=lambda tup: tup[1], reverse=True):
            outFile.write(tup[0]+','+first_rfc[tup[0]]+','+str(tup[1])+'\n')
        outFile.write('avg,,'+str(sum(vals)/len(vals))+'\n')
        outFile.write('std,,' + str(std(vals)))
    with open(outDir + '/occurence-values', 'w') as outFile:
        print(*sorted(vals), sep=' ', file=outFile)


elif case == 'quota':

    vals = list()

    with open(inputFile, 'r') as inFile:
        for row in csv.DictReader(inFile):
            if row['country'] == 'avg' or row['country'] == 'sum' or row['country'] == 'std':
                continue
            countries.add(row['country'])
        countrySuperDict = dict.fromkeys(countries)
        for country in countries:
            countrySuperDict[country] = OrderedDict.fromkeys(spans, None)
        sumDict = OrderedDict.fromkeys(spans, None)
        for s in sumDict.keys():
            sumDict[s] = list()
        inFile.seek(0)
        for row in csv.DictReader(inFile):
            if row['country'] == 'avg' or row['country'] == 'sum' or row['country'] == 'std':
                continue
            val = float(row[case])
            vals.append(val)
            countrySuperDict[row['country']][row['date']] = val
            sumDict[row['date']].append(val)

    vals.sort()

    with open(outDir + '/values', 'w') as outFile:
        print(*vals, sep=' ', file=outFile)

    with open(outDir + '/sum.csv', 'w') as outFile:
        outFile.write('date' + ',quota\n')
        for span, vals in sumDict.items():
            if vals:
                outFile.write(span+','+str(sum(vals)/len(vals))+'\n')

    for country, countryDict in countrySuperDict.items():
        outputFile = outDir + '/' + country.replace(' ', '_') + '.csv'
        with open(outputFile, 'w') as outFile:
            outFile.write('date' + ',quota\n')
            for span, quota in countryDict.items():
                if quota:
                    outFile.write(span+','+str(quota)+'\n')


# simply extract values and write in one line
elif case in ['amount', 'company']:

    vals, concentration = list(), list()

    if case == 'company':
        op = case
    else:
        op = 'country'

    with open(inputFile, 'r') as inFile:
        if inputFile.endswith('quota.csv'):
            case = 'quota'
        for row in csv.DictReader(inFile):
            if row[op] == 'avg' or row[op] == 'sum' or row[op] == 'std':
                continue
            if case == 'quota':
                vals.append(float(row['quota']))
            else:
                vals.append(int(row['amount']))
                concentration.append(row['concentration'])

    vals.sort()

    with open(outDir + '/values', 'w') as outFile:
        print(*vals, sep=' ', file=outFile)

    if case in ['amount', 'company']:
        with open(outDir + '/concentration.csv', 'w') as outFile:
            outFile.write('x,y\n0,0\n')
            for num, val in enumerate(reversed(concentration), 1):
                outFile.write(str(num * (1/len(concentration))) + ',' + val + '\n')

        dist = OrderedDict()
        lowerBound = min(vals)
        upperBound = max(vals)
        v_sum, i = 0, 0
        v = vals[0]

        for bound in range(lowerBound, upperBound, args.classinterval):
            while v <= bound:
                v_sum += 1
                i += 1
                v = vals[i]
            if v_sum > 0:
                dist[bound] = v_sum
                v_sum = 0
        dist[v] = v_sum if v_sum > 0 else 1

        outName = '/' + str(args.classinterval) + '-distribution.csv' if args.classinterval > 1 else '/distribution.csv'

        with open(outDir + outName, 'w') as outFile:
            outFile.write('amount,frequence\n')
            for k, v in dist.items():
                outFile.write(str(k) + ',' + str(v) + '\n')


else:
    print('plotbuild.py: ERR: Malformed input given, returning')
