#!/usr/bin/env python3

import csv
import sys

from os import access, R_OK, makedirs
from os.path import isfile


def get_case(filename):
    for case in ['std', 'company', 'occurence']:
        if case in filename:
            return case
    print('scatterplot.py: ERR: Unprocessable quota file')
    sys.exit(1)


# check input
case = ''
if not sys.argv[1]:
    print('scatterplot.py: ERR: No input given, returning')
    sys.exit(1)
elif len(sys.argv) != 5:
    if len(sys.argv) < 4:
        print('scatterplot.py: ERR: second file missing, returning')
    elif len(sys.argv) < 5:
        print('scatterplot.py: ERR: sum file missing, returning')
    else:
        print('scatterplot.py: ERR: too much input, returning')
    sys.exit(1)
elif isfile(sys.argv[1]) and access(sys.argv[1], R_OK) and isfile(sys.argv[2]) and access(sys.argv[2], R_OK) and isfile(sys.argv[3]) and access(sys.argv[3], R_OK) and isfile(sys.argv[4]) and access(sys.argv[4], R_OK):
    file1 = sys.argv[1]
    file2 = sys.argv[2]
    sumfile = sys.argv[3]
    colorfile = sys.argv[4]
    case1 = get_case(file1)
    case2 = get_case(file2)
else:
    print('scatterplot.py: ERR: Could not read input, returning\n\t' + sys.argv[1] + '\n\t' + sys.argv[2] + '\n\t' + sys.argv[3])
    sys.exit(1)

outDir = 'out/' + case1 + '-' + case2
makedirs(outDir, exist_ok=True)

dict1, dict2, sharedict, colordict = dict(), dict(), dict(), dict()

with open(file1, 'r') as FILE1:
    reader = csv.DictReader(FILE1)
    for row in reader:
        if row['country'] == 'avg' or row['country'] == 'sum' or row['country'] == 'std':
            continue
        dict1[row['country']] = row['quota']

with open(file2, 'r') as FILE2:
    reader = csv.DictReader(FILE2)
    for row in reader:
        if row['country'] == 'avg' or row['country'] == 'sum' or row['country'] == 'std':
            continue
        dict2[row['country']] = row['quota']

with open(sumfile, 'r') as SUMFILE:
    reader = csv.DictReader(SUMFILE)
    for row in reader:
        sharedict[row['country']] = row['share']

with open(colorfile, 'r') as COLORFILE:
    reader = csv.DictReader(COLORFILE)
    for row in reader:
        colordict[row['country']] = row['color']

with open(outDir + '.csv', 'w') as outFile:
    sortedDict = {k: v for k, v in sorted(sharedict.items(), key=lambda item: item[1], reverse=True)}
    outFile.write('country' + ',' + case1 + ',' + case2 + ',weight,color\n')
    for country, share in sortedDict.items():
        if country in colordict.keys():
            outFile.write(country + ',' + dict1[country] + ',' + dict2[country] + ',' + share + ',' + colordict[country] + '\n')
        else:
            outFile.write(country + ',' + dict1[country] + ',' + dict2[country] + ',' + share + ',gray\n')

with open(outDir + '-nonull.csv', 'w') as outFile:
    # for d in [dict1, dict2]:
    for country, val in dict2.items():
        if float(val) == 0:
            sharedict.pop(country)
    sortedDict = {k: v for k, v in sorted(sharedict.items(), key=lambda item: item[1], reverse=True)}
    outFile.write('country' + ',' + case1 + ',' + case2 + ',weight,color\n')
    for country, share in sortedDict.items():
        if country in colordict.keys():
            outFile.write(country + ',' + dict1[country] + ',' + dict2[country] + ',' + share + ',' + colordict[country] + '\n')
        else:
            outFile.write(country + ',' + dict1[country] + ',' + dict2[country] + ',' + share + ',gray\n')
