#!/usr/bin/env python3

import csv
import sys

from os import access, R_OK, makedirs
from os.path import isfile

from collections import OrderedDict

from operator import itemgetter

from numpy import std

# check input
case = ''
if not sys.argv[1]:
    print('std-quotas.py: ERR: No input given, returning')
    sys.exit(1)
elif len(sys.argv) != 3:
    if len(sys.argv) < 3:
        print('std-quotas.py: ERR: second file missing, returning')
    else:
        print('std-quotas.py: ERR: too much input, returning')
    sys.exit(1)
elif isfile(sys.argv[1]) and access(sys.argv[1], R_OK) and isfile(sys.argv[2]) and access(sys.argv[2], R_OK):
    normFile = sys.argv[1]
    stdFile = sys.argv[2]
    if 'quota' in normFile:
        sys.exit(0)
    elif 'sum' in normFile:
        case = 'amount'
    elif 'yearly' in normFile:
        case = 'year'
    elif 'monthly' in normFile:
        case = 'year-month'
else:
    print('std-quotas.py: ERR: Could not read input, returning\n\t' + sys.argv[1] + '\n\t' + sys.argv[2])
    sys.exit(1)

outDir = 'out/' + normFile.split('/')[-1][:len(normFile.split('/')[-1]) - 4]
makedirs(outDir, exist_ok=True)

with open(normFile, 'r') as RFC:
    rfcReader = csv.DictReader(RFC)
    with open(stdFile, 'r') as STANDARDS:
        stdReader = csv.DictReader(STANDARDS)

        if case == 'amount':
            tuple_list = list()
            rfcCountries, stdCountries = dict(), dict()
            for rfcRow in rfcReader:
                if rfcRow['country'] == 'avg' or rfcRow['country'] == 'sum' or rfcRow['country'] == 'std':
                    continue
                rfcCountries[rfcRow['country']] = rfcRow[case]
            for stdRow in stdReader:
                if stdRow['country'] == 'avg' or stdRow['country'] == 'sum' or stdRow['country'] == 'std':
                    continue
                stdCountries[stdRow['country']] = stdRow[case]
            for rfcCountry in rfcCountries.items():
                if int(rfcCountry[1]) > 0 and rfcCountry[0] in stdCountries:
                    quota = int(stdCountries[rfcCountry[0]]) / int(rfcCountry[1])
                else:
                    quota = 0
                tuple_list.append((rfcCountry[0], quota))

            vals = list()
            with open(outDir + '/std-quota.csv', 'w') as outFile:
                outFile.write('country,quota\n')
                for tup in sorted(tuple_list, key=itemgetter(1), reverse=True):
                    outFile.write(tup[0]+','+str(tup[1])+'\n')
                    vals.append(tup[1])
                outFile.write('avg,'+str(sum(vals)/len(vals))+'\n')
                outFile.write('std,' + str(std(vals)))
            with open(outDir + '/std-values', 'w') as outFile:
                print(*sorted(vals), sep=' ', file=outFile)

        elif case in ['year', 'year-month']:
            spans = list()
            countries = set()

            if case == 'year':
                for year in range(1968, 2020):
                    spans.append(str(year))
            else:
                for year in range(1968, 2020):
                    for month in range(1, 13):
                        spans.append(str(year) + f'-{month:02}' + '-01')

            for rfcRow in rfcReader:
                countries.add(rfcRow['country'])
            rfcCountrySuperDict = dict.fromkeys(countries)
            stdCountrySuperDict = dict.fromkeys(countries)
            for country in countries:
                rfcCountrySuperDict[country] = OrderedDict.fromkeys(spans, 0)
                stdCountrySuperDict[country] = OrderedDict.fromkeys(spans, 0)
            RFC.seek(0)
            rfcReader = csv.DictReader(RFC)
            for rfcRow in rfcReader:
                rfcCountrySuperDict[rfcRow['country']][rfcRow['date']] = rfcRow['amount']
            for stdRow in stdReader:
                stdCountrySuperDict[stdRow['country']][stdRow['date']] = stdRow['amount']

            sumDict = OrderedDict.fromkeys(spans, None)
            for s in sumDict.keys():
                sumDict[s] = list()
            for country, countryDict in rfcCountrySuperDict.items():
                outputFile = outDir + '/' + country.replace(' ', '_') + '-std-quota.csv'
                with open(outputFile, 'w') as outFile:
                    outFile.write('date' + ',quota\n')
                    for span, amount in countryDict.items():
                        stdAmount = stdCountrySuperDict[country][span]
                        if int(amount) == 0 or int(stdAmount) == 0:
                            continue
                        else:
                            outFile.write(span+','+str(int(stdAmount)/int(amount))+'\n')
                            sumDict[span].append(int(stdAmount)/int(amount))

            with open(outDir + '/sum-std-quota.csv', 'w') as outFile:
                outFile.write('date' + ',quota\n')
                for span, vals in sumDict.items():
                    if vals:
                        outFile.write(span+','+str(sum(vals)/len(vals))+'\n')
